# code from the victronenergy community pages:
# sdm630v2.zip from telekatz post Nov 10 2023
from copy import copy
import logging
import time
import device

log = logging.getLogger()

class ModbusDeviceEastron(device.ModbusDevice):

    def read_data_regs(self, regs, d):
        now = time.time()

        if all(now - r.time < r.max_age for r in regs):
            return

        start = regs[0].base
        count = regs[-1].base + regs[-1].count - start

        rr = self.modbus.read_input_registers(start, count, unit=self.unit)

        latency = time.time() - now

        if rr.isError():
            log.error('Error reading registers %#04x-%#04x: %s',
                      start, start + count - 1, rr)
            raise Exception(rr)

        for reg in regs:
            base = reg.base - start
            end = base + reg.count

            if now - reg.time > reg.max_age:
                if reg.decode(rr.registers[base:end]):
                    d[reg.name] = copy(reg) if reg.isvalid() else None
                reg.time = now

        return latency


class CustomName(device.CustomName):
    pass


class EnergyMeter(ModbusDeviceEastron, device.ModbusDevice):
    role_names = ['grid', 'pvinverter', 'genset', 'acload']
    allowed_roles = role_names
    default_role = 'grid'
    default_instance = 40
    nr_phases = None

    def position_setting_changed(self, service, path, value):
        self.dbus['/Position'] = value['Value']

    def init_device_settings(self, dbus):
        super().init_device_settings(dbus)

        self.pos_item = None
        if self.role == 'pvinverter':
            self.pos_item = self.settings.addSetting(
                self.settings_path + '/Position', 0, 0, 2,
                callback=self.position_setting_changed)

    def device_init_late(self):
        super().device_init_late()

        if self.pos_item is not None:
            self.dbus.add_path('/Position', self.pos_item.get_value(),
                               writeable=True,
                               onchangecallback=self.position_changed)

    def position_changed(self, path, val):
        if not 0 <= val <= 2:
            return False
        self.pos_item.set_value(val)
        return True
